@@ -99,6 +99,20 @@ class CORE_EXPORT QgsEditFormConfig
     };
     Q_ENUM( PythonInitCodeSource )
 
+    /**
+     * Data defined properties.
+     * Form data defined overrides are stored in a property collection
+     * and they can be retrieved using the indexes specified in this
+     * enum.
+     * \since QGIS 3.14
+     */
+    enum DataDefinedProperty
+    {
+      NoProperty = 0, //!< No property
+      AllProperties = 1, //!< All properties for item
+      Alias = 2, //!< Alias
+    };
+
     /**
      * Copy constructor
      *
@@ -159,7 +173,8 @@ class CORE_EXPORT QgsEditFormConfig
     /**
      * Set the editor widget config for a widget which is not for a simple field.
      *
-     * Example:
+     * ### Example
+     *
      * \code{.py}
      *   editFormConfig = layer.editFormConfig()
      *   editFormConfig.setWidgetConfig( 'relation_id', { 'nm-rel': 'other_relation' } )
@@ -208,7 +223,7 @@ class CORE_EXPORT QgsEditFormConfig
      * If this returns TRUE, the widget at the given index will receive its label on the previous line
      * while if it returns FALSE, the widget will receive its label on the left hand side.
      * Labeling on top leaves more horizontal space for the widget itself.
-     **/
+     */
     bool labelOnTop( int idx ) const;
 
     /**
@@ -216,7 +231,7 @@ class CORE_EXPORT QgsEditFormConfig
      * the previous line while if it is set to FALSE, the widget will receive its label
      * on the left hand side.
      * Labeling on top leaves more horizontal space for the widget itself.
-     **/
+     */
     void setLabelOnTop( int idx, bool onTop );
 
 
@@ -301,6 +316,25 @@ class CORE_EXPORT QgsEditFormConfig
      */
     explicit QgsEditFormConfig();
 
+    /**
+     * Set data defined properties for \a fieldName to \a properties
+     * \since QGIS 3.14
+     */
+    void setDataDefinedFieldProperties( const QString &fieldName, const QgsPropertyCollection &properties );
+
+    /**
+     * Returns data defined properties for \a fieldName
+     * \since QGIS 3.14
+     */
+    QgsPropertyCollection dataDefinedFieldProperties( const QString &fieldName ) const;
+
+
+    /**
+     * Returns data defined property definitions.
+     * \since QGIS 3.14
+     */
+    static const QgsPropertiesDefinition &propertyDefinitions();
+
   private:
 
     /**
@@ -314,6 +348,12 @@ class CORE_EXPORT QgsEditFormConfig
      */
     void onRelationsLoaded();
 
+    /**
+     * Used for the backwards compatibility of the api, on setting nmrel or force-suppress-popup for relations.
+     * Returns TRUE when a relation instance (the first one found) has been updated.
+     */
+    bool legacyUpdateRelationWidgetInTabs( QgsAttributeEditorContainer *container,  const QString &widgetName, const QVariantMap &config );
+
   private:
     QExplicitlySharedDataPointer<QgsEditFormConfigPrivate> d;
 
